@@ -80,6 +80,9 @@ class RemoteBatchPEGASUS:
                 sshIdentityPath,
                 gridsite,
                 pegasusSiteInfo,
+                nGpus,
+                gpn,
+                memoryMB,
                 wallTime,
                 timeHistoryLogs):
       self.logger            = logging.getLogger(__name__)
@@ -99,6 +102,9 @@ class RemoteBatchPEGASUS:
       self.pegasusVersion    = pegasusVersion
       self.pegasusHome       = pegasusHome
       self.appScriptName     = appScriptName
+      self.nGpus             = nGpus
+      self.gpn               = gpn
+      self.memoryMB          = memoryMB
       self.wallTime          = wallTime
       self.timestampStart    = timeHistoryLogs['timestampStart']
       self.timestampFinish   = timeHistoryLogs['timestampFinish']
@@ -132,6 +138,7 @@ class RemoteBatchPEGASUS:
       self.postManagerCommands      = managerInfo['postManagerCommands']
       self.pegasusTemplates         = siteInfo['pegasusTemplates']
       self.submissionScriptCommands = siteInfo['submissionScriptCommands']
+      self.timePaths                = siteInfo['timePaths']
       self.toolFilesInfo            = toolFilesInfo
       self.dockerImageInfo          = dockerImageInfo
       self.submissionScriptsInfo    = submissionScriptsInfo
@@ -300,20 +307,22 @@ class RemoteBatchPEGASUS:
       substitutions["POSTMANAGERCOMMANDS"]      = "\n".join(self.postManagerCommands)
       if self.checkDaxPath:
          if   self.checkDaxPath.endswith('.dax'):
-            substitutions["DAX"] = "--dax %s" % (self.checkDaxPath)
+            substitutions["DAX"]                = "--dax %s" % (self.checkDaxPath)
          elif self.checkDaxPath.endswith('.xml'):
-            substitutions["DAX"] = "--dax %s" % (self.checkDaxPath)
+            substitutions["DAX"]                = "--dax %s" % (self.checkDaxPath)
          elif self.checkDaxPath.endswith('.yml'):
-            substitutions["DAX"] = self.checkDaxPath
+            substitutions["DAX"]                = self.checkDaxPath
       else:
          substitutions["DAX"] = ""
+      substitutions["TIMEPATHS"]                = ' '.join(self.timePaths)
 
       template = ParameterTemplate(rawSubmissionScript)
       try:
          submissionScript = template.substitute_recur(substitutions)
       except KeyError as e:
          submissionScript = ""
-         self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@%s\n" % (e.args[0])))
+         self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@{%s}\n" % (e.args[0])))
+         self.logger.log(logging.ERROR,getLogMessage("in SubmissionScripts/Distributor/Batch/PEGASUS/serial\n"))
       except TypeError:
          submissionScript = ""
          self.logger.log(logging.ERROR,getLogMessage("Submission script substitution failed:\n%s\n" % (rawSubmissionScript)))
@@ -337,6 +346,7 @@ class RemoteBatchPEGASUS:
       substitutions["INSTANCEDIRECTORY"]    = self.instanceDirectory
       substitutions["BASESCRATCHDIRECTORY"] = os.path.basename(self.scratchDirectory)
       substitutions["SCRATCHDIRECTORY"]     = self.scratchDirectory
+      substitutions["MEMORY"]               = self.memoryMB
       if self.x509SubmitProxy != "":
          substitutions["X509SUBMITPROXY"]   = self.x509SubmitProxy
       else:
@@ -359,7 +369,8 @@ class RemoteBatchPEGASUS:
                      pegasusText = pegasusTemplate.substitute_recur(substitutions)
                   except KeyError as e:
                      pegasusText = ""
-                     self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@%s\n" % (e.args[0])))
+                     self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@{%s}\n" % (e.args[0])))
+                     self.logger.log(logging.ERROR,getLogMessage("in Pegasus %s template\n" % (templateType)))
                   except TypeError:
                      pegasusText = ""
                      self.logger.log(logging.ERROR,getLogMessage("Pegasus template substitution failed:\n%s\n" % (template)))
