@@ -29,6 +29,7 @@ import os
 import re
 import glob
 import traceback
+import json
 import logging
 
 from hubzero.submit.LogMessage import getLogJobIdMessage as getLogMessage
@@ -62,9 +63,9 @@ class TunnelsInfo:
 
    def readTunnelsInfoFile(self,
                            tunnelsInfoPath):
-      tunnelPattern   = re.compile('(\s*\[)([^\s]*)(]\s*)')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      tunnelPattern   = re.compile(r'(\s*\[)([^\s]*)(]\s*)')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       tunnelName      = ""
 
       if os.path.exists(tunnelsInfoPath):
@@ -99,23 +100,30 @@ class TunnelsInfo:
                               self.tunnels[tunnelName][key] = int(value)
                            elif isinstance(self.tunnels[tunnelName][key],dict):
                               try:
-                                 sampleKey   = self.tunnels[tunnelName][key].keys()[0]
+                                 sampleKey   = list(self.tunnels[tunnelName][key].keys())[0]
                                  sampleValue = self.tunnels[tunnelName][key][sampleKey]
                               except:
-                                 sampleKey   = "key"
-                                 sampleValue = "value"
-                              self.tunnels[tunnelName][key] = {}
-                              for e in value.split(','):
-                                 dictKey,dictValue = e.split(':')
-                                 if isinstance(sampleKey,int):
-                                    dictKey = int(dictKey)
-                                 if   isinstance(sampleValue,int):
-                                    dictValue = int(dictValue)
-                                 elif isinstance(sampleValue,float):
-                                    dictValue = float(dictValue)
-                                 elif isinstance(sampleValue,bool):
-                                    dictValue = bool(dictValue.lower() == 'true')
-                                 self.tunnels[tunnelName][key][dictKey] = dictValue
+                                 try:
+                                    self.tunnels[tunnelName][key] = json.loads(value)
+                                 except:
+                                    self.tunnels[tunnelName][key] = {}
+                                    sampleKey   = "key"
+                                    sampleValue = "value"
+                              else:
+                                 self.tunnels[tunnelName][key] = {}
+               
+                              if not self.tunnels[tunnelName][key]:
+                                 for e in value.split(','):
+                                    dictKey,dictValue = e.split(':')
+                                    if isinstance(sampleKey,int):
+                                       dictKey = int(dictKey)
+                                    if   isinstance(sampleValue,int):
+                                       dictValue = int(dictValue)
+                                    elif isinstance(sampleValue,float):
+                                       dictValue = float(dictValue)
+                                    elif isinstance(sampleValue,bool):
+                                       dictValue = bool(dictValue.lower() == 'true')
+                                    self.tunnels[tunnelName][key][dictKey] = dictValue
                            else:
                               self.tunnels[tunnelName][key] = value
                         else:
