@@ -55,7 +55,8 @@ class VenueMechanismCore:
                 timeHistoryLogs,
                 disableJobMonitoring,
                 siteInfo,
-                tapisSiteInfo,
+                tapis2SiteInfo,
+                tapis3SiteInfo,
                 fileMoverInfo,
                 toolFilesInfo,
                 dockerImageInfo,
@@ -66,6 +67,9 @@ class VenueMechanismCore:
                 nCpus,
                 nNodes,
                 ppn,
+                nGpus,
+                gpn,
+                memoryMB,
                 event):
       self.logger                = logging.getLogger(__name__)
       self.session               = session
@@ -76,7 +80,8 @@ class VenueMechanismCore:
       self.timeHistoryLogs       = copy.copy(timeHistoryLogs)
       self.disableJobMonitoring  = disableJobMonitoring
       self.siteInfo              = copy.copy(siteInfo)
-      self.tapisSiteInfo         = copy.copy(tapisSiteInfo)
+      self.tapis2SiteInfo        = copy.copy(tapis2SiteInfo)
+      self.tapis3SiteInfo        = copy.copy(tapis3SiteInfo)
       self.fileMoverInfo         = copy.copy(fileMoverInfo)
       self.toolFilesInfo         = copy.copy(toolFilesInfo)
       self.dockerImageInfo       = copy.copy(dockerImageInfo)
@@ -103,6 +108,9 @@ class VenueMechanismCore:
       self.nCpus              = nCpus
       self.nNodes             = nNodes
       self.ppn                = ppn
+      self.nGpus              = nGpus
+      self.gpn                = gpn
+      self.memoryMB           = memoryMB
       self.event              = event
 
       self.preStageJobInput = ""
@@ -357,7 +365,7 @@ class VenueMechanismCore:
 
 
    def copyJobInputs(self):
-      if self.fileMoverInfo['fileMoverType'] == 'tapis':
+      if   self.fileMoverInfo['fileMoverType'] == 'tapis2':
          self.filesStashed = True
          self.jobStatistics[self.jobIndex]['venue'] = self.venue
 
@@ -370,7 +378,59 @@ class VenueMechanismCore:
          exitStatus,stdOutput,stdError = self.executeCommand(command)
          tFinish = time.time()
          self.logger.log(logging.INFO,getLogMessage(stdOutput))
-         self.logger.log(logging.DEBUG,getLogMessage("copyJobInputs: Tapis copy took %f secs" % (tFinish-tStart)))
+#        self.logger.log(logging.DEBUG,getLogMessage("copyJobInputs: Tapis2 copy took %f secs" % (tFinish-tStart)))
+
+         if exitStatus:
+            self.filesStashed = False
+            self.jobStatistics[self.jobIndex]['exitCode'] = 11
+#           if stdOutput != "":
+#              if self.instanceId != "0":
+#                 stdFile = os.path.join(self.instanceDirectory,"%s_%s.stdout" % (self.runName,self.instanceId))
+#              else:
+#                 stdFile = os.path.join(self.instanceDirectory,"%s.stdout" % (self.runName))
+#              try:
+#                 fpStd = open(stdFile,'a')
+#                 try:
+#                    fpStd.write(stdOutput)
+#                 except (IOError,OSError):
+#                    self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (stdFile)))
+#                 finally:
+#                    fpStd.close()
+#              except (IOError,OSError):
+#                 self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (stdFile)))
+   
+            if stdError != "":
+               if self.instanceId != "0":
+                  stdFile = os.path.join(self.instanceDirectory,"%s_%s.stderr" % (self.runName,self.instanceId))
+               else:
+                  stdFile = os.path.join(self.instanceDirectory,"%s.stderr" % (self.runName))
+               try:
+                  fpStd = open(stdFile,'a')
+                  try:
+                     fpStd.write(stdError)
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (stdFile)))
+                  finally:
+                     fpStd.close()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (stdFile)))
+            self.removeJobRegistration()
+         else:
+            self.jobStatistics[self.jobIndex]['exitCode'] = 0
+      elif self.fileMoverInfo['fileMoverType'] == 'tapis3':
+         self.filesStashed = True
+         self.jobStatistics[self.jobIndex]['venue'] = self.venue
+
+         command = self.copyJobInput + " " + self.instanceDirectory + " " + \
+                                             self.fileMoverInfo['tapisStorageSystem'] + " " + \
+                                             self.instanceToken + " " + \
+                                             os.path.join(self.instanceDirectory,self.timeHistoryLogs['timestampInputStaged'])
+         self.logger.log(logging.INFO,getLogMessage("command = " + command))
+         tStart = time.time()
+         exitStatus,stdOutput,stdError = self.executeCommand(command)
+         tFinish = time.time()
+         self.logger.log(logging.INFO,getLogMessage(stdOutput))
+#        self.logger.log(logging.DEBUG,getLogMessage("copyJobInputs: Tapis3 copy took %f secs" % (tFinish-tStart)))
 
          if exitStatus:
             self.filesStashed = False
@@ -416,7 +476,7 @@ class VenueMechanismCore:
 
 
    def preStageJobInputs(self):
-      if   self.fileMoverInfo['fileMoverType'] == 'tapis':
+      if   self.fileMoverInfo['fileMoverType'] == 'tapis2':
          self.filesStashed = True
          self.jobStatistics[self.jobIndex]['venue'] = self.venue
 
@@ -429,7 +489,59 @@ class VenueMechanismCore:
          exitStatus,stdOutput,stdError = self.executeCommand(command)
          tFinish = time.time()
          self.logger.log(logging.INFO,getLogMessage(stdOutput))
-         self.logger.log(logging.DEBUG,getLogMessage("preStageJobInputs: Tapis prestage took %f secs" % (tFinish-tStart)))
+#        self.logger.log(logging.DEBUG,getLogMessage("preStageJobInputs: Tapis2 prestage took %f secs" % (tFinish-tStart)))
+
+         if exitStatus:
+            self.filesStashed = False
+            self.jobStatistics[self.jobIndex]['exitCode'] = 11
+#           if stdOutput != "":
+#              if self.instanceId != "0":
+#                 stdFile = os.path.join(self.instanceDirectory,"%s_%s.stdout" % (self.runName,self.instanceId))
+#              else:
+#                 stdFile = os.path.join(self.instanceDirectory,"%s.stdout" % (self.runName))
+#              try:
+#                 fpStd = open(stdFile,'a')
+#                 try:
+#                    fpStd.write(stdOutput)
+#                 except (IOError,OSError):
+#                    self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (stdFile)))
+#                 finally:
+#                    fpStd.close()
+#              except (IOError,OSError):
+#                 self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (stdFile)))
+   
+            if stdError != "":
+               if self.instanceId != "0":
+                  stdFile = os.path.join(self.instanceDirectory,"%s_%s.stderr" % (self.runName,self.instanceId))
+               else:
+                  stdFile = os.path.join(self.instanceDirectory,"%s.stderr" % (self.runName))
+               try:
+                  fpStd = open(stdFile,'a')
+                  try:
+                     fpStd.write(stdError)
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (stdFile)))
+                  finally:
+                     fpStd.close()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (stdFile)))
+            self.removeJobRegistration()
+         else:
+            self.jobStatistics[self.jobIndex]['exitCode'] = 0
+      elif self.fileMoverInfo['fileMoverType'] == 'tapis3':
+         self.filesStashed = True
+         self.jobStatistics[self.jobIndex]['venue'] = self.venue
+
+         command = self.preStageJobInput + " " + self.instanceDirectory + " " + \
+                                                 self.fileMoverInfo['preStageDirectory'] + " " + \
+                                                 self.instanceToken + " " + \
+                                                 os.path.join(self.instanceDirectory,self.timeHistoryLogs['timestampInputStaged'])
+         self.logger.log(logging.INFO,getLogMessage("command = " + command))
+         tStart = time.time()
+         exitStatus,stdOutput,stdError = self.executeCommand(command)
+         tFinish = time.time()
+         self.logger.log(logging.INFO,getLogMessage(stdOutput))
+#        self.logger.log(logging.DEBUG,getLogMessage("preStageJobInputs: Tapis3 prestage took %f secs" % (tFinish-tStart)))
 
          if exitStatus:
             self.filesStashed = False
@@ -527,10 +639,13 @@ class VenueMechanismCore:
 
  
    def stashJobInputs(self):
+#     self.logger.log(logging.DEBUG,getLogMessage("stashJobInputs: fileMoverType: %s" % (self.fileMoverInfo['fileMoverType'])))
       if self.fileMoverInfo['preStageDirectory']:
          filesStashed = self.preStageJobInputs()
       else:
          filesStashed = self.copyJobInputs()
+      if not filesStashed:
+         self.logger.log(logging.ERROR,getLogMessage("stashJobInputs: failed"))
 
       return(filesStashed)
 
@@ -538,7 +653,22 @@ class VenueMechanismCore:
    def stageJob(self):
       exitStatus = 0
 
-      if   self.fileMoverInfo['fileMoverType'] == 'tapis':
+#     self.logger.log(logging.DEBUG,getLogMessage("stageJob: fileMoverType: %s" % (self.fileMoverInfo['fileMoverType'])))
+      if   self.fileMoverInfo['fileMoverType'] == 'tapis2':
+         self.jobStaged = True
+         self.jobStatistics[self.jobIndex]['exitCode'] = 2
+         self.remoteJobMonitor.postJobStaging(self.session,self.instanceToken,self.wsJobId,
+                                              self.siteInfo['siteName'],self.siteInfo['identityManagers'],
+                                              self.siteMonitorDesignator,
+                                              self.fileMoverInfo['preStageDirectory'],self.fileMoverInfo['tapisStorageSystem'],
+                                              str(self.hubUserId),
+                                              self.enteredCommand,
+                                              self.scratchDirectory,
+                                              self.localJobId,self.instanceId,self.destination,
+                                              self.runName,self.nCpus,self.distributorPid,
+                                              self.jobStatistics[self.jobIndex])
+         self.jobStatistics[self.jobIndex]['exitCode'] = exitStatus
+      elif self.fileMoverInfo['fileMoverType'] == 'tapis3':
          self.jobStaged = True
          self.jobStatistics[self.jobIndex]['exitCode'] = 2
          self.remoteJobMonitor.postJobStaging(self.session,self.instanceToken,self.wsJobId,
@@ -567,6 +697,9 @@ class VenueMechanismCore:
                                               self.jobStatistics[self.jobIndex])
          self.jobStatistics[self.jobIndex]['exitCode'] = exitStatus
 
+      if not self.jobStaged:
+         self.logger.log(logging.ERROR,getLogMessage("stageJob: failed"))
+
       return(self.jobStaged)
 
 
